from raga.validators.dataset_creds_validations import DatasetCredsValidator
from raga.utils import HTTPClient

class DatasetCreds:
    def __init__(self, arn:str="", region:str=""):
        self.arn = arn
        self.region = region


import ast
import random
from raga import *
import pandas as pd
import datetime

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def annotation_v1(row):
    AnnotationsV1 = ImageDetectionObject()
    AnnotationsV1.add(eval(row["AnnotationsV1"]))
    return AnnotationsV1

def model_inferences(row):
    ModelInferences = ImageDetectionObject()
    for detection in row["ModelInferences"]:
        ModelInferences.add(ObjectDetection(Id=detection['Id'], ClassId=detection['ClassId'], ClassName=detection['ClassName'], Confidence=detection['Confidence'], BBox= detection['BBox'], Format="xywh_normalized"))
    return ModelInferences

def imag_vectors_m1(row):
    ImageVectorsM1 = ImageEmbedding()
    row = ast.literal_eval(row["ROIVectorsM1"])
    for embedding in row:
        ImageVectorsM1.add(embedding)
    return ImageVectorsM1

def csv_parser(csv_file):
    df = pd.read_csv(csv_file).head(1)
    data_frame = pd.DataFrame()
    data_frame["ImageId"] = df["ImageId"].apply(lambda x: x)
    data_frame["ImageUri"] = df["SourceLink"].apply(lambda x: x)
    data_frame["SourceLink"] = df["SourceLink"].apply(lambda x: x)
    data_frame["TimeOfCapture"] = df.apply(lambda row: TimeStampElement(get_timestamp_x_hours_ago(row.name)), axis=1)
    data_frame["Reflection"] = df.apply(lambda row: random.choice(["Yes", "No"]), axis=1)
    data_frame["Overlap"] = df.apply(lambda row: random.choice(["Yes", "No"]), axis=1)
    data_frame["CameraAngle"] = df.apply(lambda row: random.choice(["Yes", "No"]), axis=1)
    data_frame["AnnotationsV1"] = df.apply(annotation_v1, axis=1)
    data_frame["ROIVectorsM1"] = df.apply(imag_vectors_m1, axis=1)
    return data_frame

# pd_data_frame = csv_parser("./assets/combined_gt.csv")
pd_data_frame = csv_parser("./assets/combined_pred.csv")

schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement())
schema.add("ImageUri", ImageUriSchemaElement())
schema.add("TimeOfCapture", TimeOfCaptureSchemaElement())
schema.add("SourceLink", FeatureSchemaElement())
schema.add("Reflection", AttributeSchemaElement())
schema.add("CameraAngle", AttributeSchemaElement())
schema.add("Overlap", AttributeSchemaElement())
schema.add("AnnotationsV1", InferenceSchemaElement(model="GT"))
schema.add("ROIVectorsM1", ImageEmbeddingSchemaElement(model="imageModel"))

run_name = f"run-loader-ai-dash-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"

# create test_session object of TestSession instance
test_session = TestSession(project_name="testingProject", run_name= run_name, access_key="LGXJjQFD899MtVSrNHGH", secret_key="TC466Qu9PhpOjTuLu5aGkXyGbM7SSBeAzYH6HpcP", host="http://3.111.106.226:8080")


cred = DatasetCreds(region="us-east-1")

#create test_ds object of Dataset instance
test_ds = Dataset(test_session=test_session, 
                  name="drift_ai_dash_pred_object_detection_ds", 
                  type=DATASET_TYPE.IMAGE,
                  data=pd_data_frame, 
                  schema=schema,
                  creds=cred)

#load schema and pandas data frame
test_ds.load()


import json
import os
import zipfile
from raga import TestSession, Filter
from raga import spinner

def lightmetrics_inference_generator(test_session:TestSession, dataset_name: str, model_name:str, model_inference_col_name:str, event_inference_col_name:str, filter:Filter):
    from raga.constants import INVALID_RESPONSE, INVALID_RESPONSE_DATA, REQUIRED_ARG_V2
    dataset_id = lightmetrics_inference_generator_validation(test_session, dataset_name, model_name, model_inference_col_name, event_inference_col_name, filter)
    payload = {
            "datasetId": dataset_id,
            "experimentId": test_session.experiment_id,
            "model": model_name,
            "modelInferenceColName": model_inference_col_name,
            "eventInferenceColName": event_inference_col_name,
            'filter':filter.get()
        }
    res_data = test_session.http_client.post(f"api/experiment/test/lm/inferenceGenerator", data=payload, headers={"Authorization": f'Bearer {test_session.token}'})

    if not isinstance(res_data, dict):
        spinner.stop()
        raise ValueError(INVALID_RESPONSE)
    spinner.stop()

def lightmetrics_inference_generator_validation(test_session:TestSession, dataset_name:str, model_name:str, model_inference_col_name:str, event_inference_col_name:str, filter:Filter):
    from raga.constants import INVALID_RESPONSE, INVALID_RESPONSE_DATA, REQUIRED_ARG_V2

    assert isinstance(test_session, TestSession), f"{REQUIRED_ARG_V2.format('test_session', 'instance of the TestSession')}"
    assert isinstance(dataset_name, str) and dataset_name, f"{REQUIRED_ARG_V2.format('dataset_name', 'str')}"
    res_data = test_session.http_client.get(f"api/dataset?projectId={test_session.project_id}&name={dataset_name}", headers={"Authorization": f'Bearer {test_session.token}'})

    if not isinstance(res_data, dict):
            raise ValueError(INVALID_RESPONSE)
    
    dataset_id = res_data.get("data", {}).get("id")

    if not dataset_id:
        raise KeyError(INVALID_RESPONSE_DATA)
    assert isinstance(model_name, str) and model_name, f"{REQUIRED_ARG_V2.format('model_name', 'str')}"
    assert isinstance(model_inference_col_name, str) and model_inference_col_name, f"{REQUIRED_ARG_V2.format('model_inference_col_name', 'str')}"
    assert isinstance(event_inference_col_name, str) and event_inference_col_name, f"{REQUIRED_ARG_V2.format('event_inference_col_name', 'str')}"
    assert isinstance(filter, Filter) and filter, f"{REQUIRED_ARG_V2.format('filter', 'instance of the Filter')}"
    return dataset_id

def lightmetrics_model_upload(test_session:TestSession, file_path:str, name:str, version:str, override:bool=True):
    from raga.constants import INVALID_RESPONSE, INVALID_RESPONSE_DATA, REQUIRED_ARG_V2
    assert isinstance(file_path, str) and file_path, f"{REQUIRED_ARG_V2.format('file_path', 'str')}"
    assert isinstance(name, str) and name, f"{REQUIRED_ARG_V2.format('name', 'str')}"
    assert isinstance(version, str) and version, f"{REQUIRED_ARG_V2.format('version', 'str')}"
    
    if not os.path.exists(file_path):
        raise FileNotFoundError(f"{file_path} does not exist.")
    if not file_path.lower().endswith(".zip"):
        raise TypeError("File mush be zip")
    model_zip_validation(file_path, name)
    

    payload = {
            "projectId": test_session.project_id,
            "modelName": name,
            "modelVersion": version
        }
    
    res_data = test_session.http_client.post(f"api/model-upload", data=payload, headers={"Authorization": f'Bearer {test_session.token}'}, file=file_path)

    if not isinstance(res_data, dict):
        spinner.stop()
        raise ValueError(INVALID_RESPONSE)
    spinner.stop()
    
def model_zip_validation(path, model_name):
    with zipfile.ZipFile(path, "r") as zip_file:
        all_names = zip_file.namelist()
        directories = [name for name in all_names if name.endswith("/") and '/' in name]
        if directories:
            directories.sort(key=lambda x: x.count('/'))
            model_dir_name = directories[0].split('/')[0]
            if model_dir_name != model_name:
                raise ValueError(f"The model name provided does not correspond to the directory contained in the ZIP file.")
        else:
            raise FileNotFoundError(f"{path} is empty.")
        config_json_path = f"{model_dir_name}/configs.json"
        if config_json_path not in all_names:
            raise FileNotFoundError(f"{config_json_path} not found.")
        with zip_file.open(config_json_path) as config_file:
            config_data = json.load(config_file)
        config_model_name = next(iter(config_data))
        if config_model_name != model_name:
            raise ValueError(f"The model name provided does not correspond to the config file model name.")
        model_path = f"{model_dir_name}/Models/"
        if model_path not in all_names:
            raise FileNotFoundError(f"{model_path} not found.")
        return 1
            


import pathlib
import random
from raga import *
import pandas as pd
import datetime
import numpy as np

weather_data = {
    "overcast": "overcast",
    "fog": "fog",
    "cloudy": "overcast",
    "sunny": "clear",
    "clear": "clear",
    "snow": "snow",
    "rain": "rain",
    "partly cloudy": "overcast",
    "light rain": "rain",
    "patchy light rain with thunder": "rain",
    "patchy light rain": "rain",
    "light drizzle": "rain",
    "light freezing rain": "rain",
    "thundery outbreaks possible": "overcast",
    "patchy rain possible": "overcast",
    "patchy light drizzle": "rain",
    "heavy freezing drizzle":"rain",
    "moderate snow": "snow",
    "moderate rain at times": "rain",
    "moderate rain": "rain",
    "moderate or heavy snow showers": "snow",
    "moderate or heavy rain shower": "rain",
    "moderate or heavy rain with thunder":"rain",
    "moderate or heavy freezing rain":"rain",
    "heavy rain":"rain",
    "heavy snow":"snow",
    "mist": "fog",
    "freezing fog":"fog",
    "light snow": "snow",
    "light snow showers": "snow",
    "light sleet showers": "snow",
    "light sleet": "snow",
    "light rain shower": "rain",
    "blowing snow": "snow",
    "patchy light snow":"snow",
    "patchy moderate snow":"snow",
    "ice pellets":"snow",
    "blizzard":"snow",
    "":""
}

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def img_url(x):
    return StringElement(f"https://ragacloudstorage.s3.ap-south-1.amazonaws.com/1/StopSign_Part1_event.json/data_points/{pathlib.Path(x).stem}/{x}")

def event_a_inference(row):
    detections = EventDetectionObject()
    start_frame = row["model_1_outputs"][0]["frame_id"]
    end_frame = row["model_1_outputs"][-1]["frame_id"]
    for index, frame in enumerate(row["event_1_outputs"]):
        for detection in frame["detections"]:
            for ind, count in enumerate(range(int(detection["count"]))):
                id = ind+1
                detections.add(EventDetection(Id=id, StartFrame=start_frame, EndFrame=end_frame, EventType=detection["class"], Confidence=detection["confidence"]))
    return detections


def event_b_inference(row):
    detections = EventDetectionObject()
    start_frame = row["model_2_outputs"][0]["frame_id"]
    end_frame = row["model_2_outputs"][-1]["frame_id"]
    for index, frame in enumerate(row["event_2_outputs"]):
        for detection in frame["detections"]:
            for ind, count in enumerate(range(int(detection["count"]))):
                id = ind+1
                detections.add(EventDetection(Id=id, StartFrame=start_frame, EndFrame=end_frame, EventType=detection["class"], Confidence=detection["confidence"]))
    return detections

def model_a_video_inference(row):
    model_a_inference = VideoDetectionObject()
    for index, frame in enumerate(row["model_1_outputs"]):
        detections = ImageDetectionObject()
        for index, detection in enumerate(frame["detections"]):
            id = index+1
            detections.add(ObjectDetection(Id=id, Format="xywh_normalized", Confidence=detection["confidence"], ClassId=0, ClassName=detection["class"], BBox=detection["bbox"]))
        model_a_inference.add(VideoFrame(frameId=frame["frame_id"], timeOffsetMs=float(frame["time_offset_ms"])*1000, detections=detections))

    return model_a_inference


def model_b_video_inference(row):
    model_a_inference = VideoDetectionObject()
    for index, frame in enumerate(row["model_2_outputs"]):
        detections = ImageDetectionObject()
        for index, detection in enumerate(frame["detections"]):
            id = index+1
            detections.add(ObjectDetection(Id=id, Format="xywh_normalized", Confidence=detection["confidence"], ClassId=0, ClassName=detection["class"], BBox=detection["bbox"]))
        model_a_inference.add(VideoFrame(frameId=frame["frame_id"], timeOffsetMs=float(frame["time_offset_ms"])*1000, detections=detections))

    return model_a_inference


def model_image_inference(row):
    AnnotationsV1 = ImageDetectionObject()
    for index, detection in enumerate(row["detections"]):
        AnnotationsV1.add(ObjectDetection(Id=detection["Id"], ClassId=0, ClassName=detection['ClassName'], Confidence=detection['Confidence'], BBox= detection['BBox'], Format=detection['Format']))
    return AnnotationsV1

def generate_random_list():
    """
    Generate a random list of specified length containing floating-point numbers.

    Args:
        length (int): The number of elements in the list.

    Returns:
        A list of random floating-point numbers.
    """
    list_data = [random.choice([random.uniform(1, 20), random.uniform(1, 20), random.uniform(-10, 20)]), random.choice([random.uniform(1, 20), random.uniform(1, 20), random.uniform(-10, 20)]), random.choice([random.uniform(1, 20), random.uniform(1, 20), random.uniform(-10, 20)])]
    embeddings = ImageEmbedding()
    for embedding in list_data:
        # print(embedding)
        embeddings.add(Embedding(int(embedding)))
    return embeddings


def json_parser(event_1, event_2, model_1, model_2):
    event_1_df = pd.read_json(event_1)
    event_2_df = pd.read_json(event_2)
    model_1_df = pd.read_json(model_1)
    model_2_df = pd.read_json(model_2)

    event_1_df_exploded = event_1_df.explode('inputs')
    event_2_df_exploded = event_2_df.explode('inputs')
    model_1_df_exploded = model_1_df.explode('inputs')
    model_2_df_exploded = model_2_df.explode('inputs')

    attributes = event_2_df["attributes"].apply(pd.Series)

    event_2_df_exploded = pd.concat([event_2_df_exploded, attributes], axis=1)
    event_1_df_exploded.rename(columns={"outputs": "event_1_outputs"}, inplace=True)
    event_2_df_exploded.rename(columns={"outputs": "event_2_outputs"}, inplace=True)
    model_1_df_exploded.rename(columns={"outputs": "model_1_outputs"}, inplace=True)
    model_2_df_exploded.rename(columns={"outputs": "model_2_outputs"}, inplace=True)
    merged_df = pd.merge(event_1_df_exploded, event_2_df_exploded, on='inputs')
    merged_df = pd.merge(merged_df, model_1_df_exploded, on='inputs')
    merged_df = pd.merge(merged_df, model_2_df_exploded, on='inputs', suffixes=[None, "_model_2"])
    data_frame = pd.DataFrame()
    data_frame["videoId"] = merged_df["inputs"].apply(lambda x: StringElement(pathlib.Path(x).stem))
    data_frame["videoUrl"] = merged_df["inputs"].apply(lambda x: img_url(x))
    data_frame["timeOfCapture"] = merged_df.apply(lambda row: TimeStampElement(get_timestamp_x_hours_ago(row.name)), axis=1)
    data_frame["sourceLink"] = merged_df["inputs"].apply(lambda x: StringElement(x))
    data_frame["dutyType"] = merged_df["dutyType"].apply(lambda x: StringElement(x))
    data_frame["time_of_day"] = merged_df["time_of_day"].apply(lambda x: StringElement(x))
    data_frame["weather"] = merged_df["weather"].apply(lambda x: StringElement(weather_data[str(x).lower()]))
    data_frame["scene"] = merged_df["scene"].apply(lambda x: StringElement(x))
    data_frame["tags"] = merged_df["tags"].apply(lambda x: StringElement(x))
    data_frame["Complex-America-Stop-Event"] = merged_df.apply(event_a_inference, axis=1)
    data_frame["Production-America-Stop-Event"] = merged_df.apply(event_b_inference, axis=1)
    data_frame["Complex-America-Stop-Model"] = merged_df.apply(model_a_video_inference, axis=1)
    data_frame["Production-America-Stop-Model"] = merged_df.apply(model_b_video_inference, axis=1)
    return data_frame

def make_image_df(video_df:pd.DataFrame):
    df = data_frame_extractor(video_df)
    videos = df["videoId"]
    complex_model = df['Complex-America-Stop-Model']
    production_model = df['Production-America-Stop-Model']
    data_frame_list = []
    for index, output in enumerate(complex_model):
        frames  = output['frames']
        video = videos[index]
        complex_model_outputs = complex_model[index]['frames']
        production_model_outputs = production_model[index]['frames']
        for frame_index, frame in enumerate(frames):
            complex_detection = complex_model_outputs[frame_index]
            production_detection = production_model_outputs[frame_index]
            data_frame_list.append({
                "imageId":StringElement(f"img_{video}_{str(frame.get('frameId')).zfill(8)}"),
                "frame":StringElement(frame.get('frameId')),
                "videoId":StringElement(video),
                "imageUrl":StringElement(f"https://ragacloudstorage.s3.ap-south-1.amazonaws.com/1/StopSign_Part1_event.json/data_points/{video}/images/img_{video}_{str(frame.get('frameId')).zfill(8)}.jpg"),
                "timeOfCapture":TimeStampElement(get_timestamp_x_hours_ago(frame.get('frameId'))),
                "sourceLink":StringElement(f"img_{video}_{str(frame.get('frameId')).zfill(8)}"),
                "dutyType":StringElement(df["dutyType"][index]),
                "time_of_day":StringElement(df["time_of_day"][index]),
                "weather":StringElement(weather_data[str(df["weather"][index]).lower()]),
                "scene":StringElement(df["scene"][index]),
                "tags":StringElement(df["tags"][index]),
                "Complex-America-Stop-Model": model_image_inference(complex_detection),
                "Production-America-Stop-Model": model_image_inference(production_detection)
            })

    return pd.DataFrame(data_frame_list)

pd_video_data_frame = json_parser("./assets/Complex-America-Stop-Event.json", "./assets/Production-America-Stop-Event.json", "./assets/Complex-America-Stop-Model.json", "./assets/Production-America-Stop-Model.json")
# # # data_frame_extractor(pd_video_data_frame).to_csv("assets/event_ds_10.csv", index=False)

# pd_image_data_frame = make_image_df(pd_video_data_frame)

# # # # data_frame_extractor(pd_image_data_frame).to_csv("assets/event_ds_pd_image_data_frame_1.csv", index=False)

run_name = f"lm_video_loader_failure_mode_analysis_object_detection-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"


# def count_rows_and_chunk(df, chunk_size):
#     """
#     Count the rows of a Pandas DataFrame and break it into smaller chunks.

#     Parameters:
#         df (pd.DataFrame): The input Pandas DataFrame.
#         chunk_size (int): The size of each chunk.

#     Returns:
#         (int, list of pd.DataFrame): A tuple containing the total row count of the DataFrame
#                                      and a list of DataFrame chunks.
#     """
#     total_rows = len(df)
#     num_chunks = (total_rows + chunk_size - 1) // chunk_size  # Calculate the number of chunks needed
#     chunks = [df[i * chunk_size:(i + 1) * chunk_size] for i in range(num_chunks)]
#     return total_rows, chunks


# chunk_size = 30000  # Maximum chunk size

# total_rows, chunks = count_rows_and_chunk(pd_image_data_frame, chunk_size)
# print(f"Total Rows: {total_rows}")
# for i, chunk in enumerate(chunks):
#    chunk.to_pickle(f"./assets/Complex-production-America-Stop-Sign/ss_data_{str(i).zfill(8)}.pkl")



# image_ds_schema = RagaSchema()
# image_ds_schema.add("imageId", PredictionSchemaElement())
# image_ds_schema.add("frame", FrameSchemaElement())
# image_ds_schema.add("videoId", ParentSchemaElement())
# image_ds_schema.add("imageUrl", ImageUriSchemaElement())
# image_ds_schema.add("timeOfCapture", TimeOfCaptureSchemaElement())
# image_ds_schema.add("sourceLink", FeatureSchemaElement())
# image_ds_schema.add("weather", AttributeSchemaElement())
# image_ds_schema.add("time_of_day", AttributeSchemaElement())
# image_ds_schema.add("scene", AttributeSchemaElement())
# image_ds_schema.add("Complex-America-Stop-Model", InferenceSchemaElement(model="Complex-America-Stop-Event"))
# image_ds_schema.add("Production-America-Stop-Model", InferenceSchemaElement(model="Production-America-Stop-Event"))


# create test_session object of TestSession instance
# test_session = TestSession(project_name="testingProject", run_name= run_name)

# creds = DatasetCreds(arn="arn:aws:iam::527593518644:role/raga-importer")
#create test_ds object of Dataset instance
# image_ds = Dataset(test_session=test_session,
#                   name="stopsign-event-image-ds-v8",
#                   type=DATASET_TYPE.IMAGE,
#                   data=pd_image_data_frame,
#                   schema=image_ds_schema,
#                   creds=creds)

# #load schema and pandas data frame
# image_ds.load()

# model_exe_fun = ModelExecutorFactory().get_model_executor(test_session=test_session, 
#                                                           model_name="Lightmetrics Model", 
#                                                           version="0.1.1")

# model_exe_fun.execute(init_args={"device": "cpu", "frame_sampling_rate":30}, 
#                            execution_args={"input_columns":{"img_paths":"imageUrl"}, 
#                                            "output_columns":{"embedding":"imageEmbedding"},
#                                            "column_schemas":{"embedding":ImageEmbeddingSchemaElement(model="Lightmetrics Model")}}, 
#                            data_frame=image_ds)

# image_ds.load()


# video_schema = RagaSchema()
# video_schema.add("videoId", PredictionSchemaElement())
# video_schema.add("videoUrl", ImageUriSchemaElement())
# video_schema.add("timeOfCapture", TimeOfCaptureSchemaElement())
# video_schema.add("sourceLink", FeatureSchemaElement())
# video_schema.add("dutyType", AttributeSchemaElement())
# video_schema.add("time_of_day", AttributeSchemaElement())
# video_schema.add("weather", AttributeSchemaElement())
# video_schema.add("scene", AttributeSchemaElement())
# video_schema.add("tags", AttributeSchemaElement())
# video_schema.add("Complex-America-Stop-Event", EventInferenceSchemaElement(model="Complex-America-Stop-Event"))
# video_schema.add("Production-America-Stop-Event", EventInferenceSchemaElement(model="Production-America-Stop-Event"))

# #create test_ds object of Dataset instance
# video_ds = Dataset(test_session=test_session,
#                   name="stopsign-event-video-ds-full-v5",
#                   type=DATASET_TYPE.VIDEO,
#                   data=pd_video_data_frame,
#                   schema=video_schema,
#                   creds=creds,
#                   parent_dataset="stopsign-event-img-ds-full-v5")
# #load schema and pandas data frame
# video_ds.load()

run_name = f"lm_video_loader_failure_mode_analysis_object_detection-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"
test_session = TestSession(project_name="testingProject", run_name= run_name)


paths = []
folder_path = "./assets/Complex-production-America-Stop-Sign-3"
# Check if the folder path exists
if os.path.exists(folder_path) and os.path.isdir(folder_path):
    # Get a list of all files in the folder
    file_list = [f for f in os.listdir(folder_path) if os.path.isfile(os.path.join(folder_path, f))]
    
    # Print the list of files with their full paths
    for file_name in file_list:
        full_path = os.path.join(folder_path, file_name)
        paths.append(full_path)
else:
    print(f"The folder path '{folder_path}' does not exist or is not a directory.")
paths = sorted(paths)
# print(paths)
for path in paths:
    image_ds_schema = RagaSchema()
    image_ds_schema.add("imageId", PredictionSchemaElement())
    image_ds_schema.add("frame", FrameSchemaElement())
    image_ds_schema.add("videoId", ParentSchemaElement())
    image_ds_schema.add("imageUrl", ImageUriSchemaElement())
    image_ds_schema.add("timeOfCapture", TimeOfCaptureSchemaElement())
    image_ds_schema.add("sourceLink", FeatureSchemaElement())
    image_ds_schema.add("weather", AttributeSchemaElement())
    image_ds_schema.add("time_of_day", AttributeSchemaElement())
    image_ds_schema.add("scene", AttributeSchemaElement())
    image_ds_schema.add("Complex-America-Stop-Model", InferenceSchemaElement(model="Complex-America-Stop-Model"))
    image_ds_schema.add("Production-America-Stop-Model", InferenceSchemaElement(model="Production-America-Stop-Model"))

    creds = DatasetCreds(region="ap-south-1")
    # create test_ds object of Dataset instance
    df = pd.read_pickle(path)
    print("PAAAAAATH", path)
    test_ds = Dataset(test_session=test_session, 
                      name="stopsign-event-img-ds-full-v5", 
                      type=DATASET_TYPE.IMAGE,
                      data=df, 
                      schema=image_ds_schema, 
                      creds=creds)

    #load schema and pandas data frame
    test_ds.load()

    model_exe_fun = ModelExecutorFactory().get_model_executor(test_session=test_session, 
                                                          model_name="Lightmetrics Embedding Model", 
                                                          version="0.1.2")

    df = model_exe_fun.execute(init_args={"device": "cpu", "frame_sampling_rate":30}, 
                            execution_args={"input_columns":{"img_paths":"imageUrl"}, 
                                            "output_columns":{"embedding":"imageEmbedding"},
                                            "column_schemas":{"embedding":ImageEmbeddingSchemaElement(model="Lightmetrics Embedding Model")}}, 
                            data_frame=test_ds)

    print(df)
    test_ds.load()



import pathlib
from raga import *
import pandas as pd
import datetime

weather_data = {
    "overcast": "overcast",
    "fog": "fog",
    "cloudy": "overcast",
    "sunny": "clear",
    "clear": "clear",
    "snow": "snow",
    "rain": "rain",
    "partly cloudy": "overcast",
    "light rain": "rain",
    "patchy light rain with thunder": "rain",
    "patchy light rain": "rain",
    "light drizzle": "rain",
    "light freezing rain": "rain",
    "thundery outbreaks possible": "overcast",
    "patchy rain possible": "overcast",
    "patchy light drizzle": "rain",
    "heavy freezing drizzle":"rain",
    "moderate snow": "snow",
    "moderate rain at times": "rain",
    "moderate rain": "rain",
    "moderate or heavy snow showers": "snow",
    "moderate or heavy rain shower": "rain",
    "moderate or heavy rain with thunder":"rain",
    "moderate or heavy freezing rain":"rain",
    "heavy rain":"rain",
    "heavy snow":"snow",
    "mist": "fog",
    "freezing fog":"fog",
    "light snow": "snow",
    "light snow showers": "snow",
    "light sleet showers": "snow",
    "light sleet": "snow",
    "light rain shower": "rain",
    "blowing snow": "snow",
    "patchy light snow":"snow",
    "patchy moderate snow":"snow",
    "ice pellets":"snow",
    "blizzard":"snow",
    "":"NA"
}

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def img_url(x):
    return StringElement(f"https://ragacloudstorage.s3.ap-south-1.amazonaws.com/1/StopSign_Part1_event.json/data_points/{pathlib.Path(x).stem}/{x}")

def json_parser(event_1):
    event_1_df = pd.read_json(event_1)

    event_1_df_exploded = event_1_df.explode('inputs')

    attributes = event_1_df["attributes"].apply(pd.Series)

    event_1_df_exploded = pd.concat([event_1_df_exploded, attributes], axis=1)
    data_frame = pd.DataFrame()
    data_frame["videoId"] = event_1_df_exploded["inputs"].apply(lambda x: StringElement(pathlib.Path(x).stem))
    data_frame["videoUrl"] = event_1_df_exploded["inputs"].apply(lambda x: img_url(x))
    data_frame["timeOfCapture"] = event_1_df_exploded.apply(lambda row: TimeStampElement(get_timestamp_x_hours_ago(row.name)), axis=1)
    data_frame["dutyType"] = event_1_df_exploded["dutyType"].apply(lambda x: StringElement(x))
    data_frame["time_of_day"] = event_1_df_exploded["time_of_day"].apply(lambda x: StringElement(x))
    data_frame["weather"] = event_1_df_exploded["weather"].apply(lambda x: StringElement(weather_data[str(x).lower()]))
    data_frame["scene"] = event_1_df_exploded["scene"].apply(lambda x: StringElement(x))
    data_frame["tags"] = event_1_df_exploded["tags"].apply(lambda x: StringElement(x))
    # return data_frame
    return data_frame.head(50)


pd_video_data_frame = json_parser("./assets/Complex-America-Stop-Event.json")

# print(pd_video_data_frame)
# # data_frame_extractor(pd_video_data_frame).to_csv("assets/event_ds_pd_10.csv", index=False)


schema = RagaSchema()
schema.add("videoId", PredictionSchemaElement())
schema.add("videoUrl", ImageUriSchemaElement())
schema.add("timeOfCapture", TimeOfCaptureSchemaElement())
schema.add("dutyType", AttributeSchemaElement())
schema.add("time_of_day", AttributeSchemaElement())
schema.add("weather", AttributeSchemaElement())
schema.add("scene", AttributeSchemaElement())
schema.add("tags", AttributeSchemaElement())

run_name = f"lm_video_loader_failure_mode_analysis_object_detection-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"


# # create test_session object of TestSession instance
test_session = TestSession(project_name="testingProject", run_name= run_name, access_key="LGXJjQFD899MtVSrNHGH", secret_key="TC466Qu9PhpOjTuLu5aGkXyGbM7SSBeAzYH6HpcP", host="http://3.111.106.226:8080")

creds = DatasetCreds(arn="arn:aws:iam::527593518644:role/raga-importer")

# #create test_ds object of Dataset instance
video_ds = Dataset(test_session=test_session,
                  name="test-lm-loader-3-nov-v3",
                  type=DATASET_TYPE.VIDEO,
                  data=pd_video_data_frame,
                  schema=schema,
                  creds=creds)
#load schema and pandas data frame
video_ds.lightmetrics_data_upload()


import pathlib
from raga import *
import pandas as pd
import datetime


label_to_classname = {
    1: "no_data",
    2: "water",
    3: "trees",
    4: "grass",
    5: "flooded vegetation",
    6: "crops",
    7: "scrub",
    8: "built_area",
    9: "bare_ground",
    10: "snow_or_ice",
    11: "clouds",
}

def generate_fake_image_embedding(embedding_dimension):
    """
    Generate a single fake image embedding.

    Args:
        embedding_dimension (int): Dimensionality of the embedding.

    Returns:
        A fake image embedding as a NumPy array.
    """
    # Generate a random fake image embedding vector
    fake_embedding = np.random.rand(embedding_dimension)
    embeddings = ImageEmbedding()
    for embedding in fake_embedding:
        # print(embedding)
        embeddings.add(Embedding(embedding))
    return embeddings

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def image_url(x):
    return f"https://ragatesitng-dev-storage.s3.ap-south-1.amazonaws.com/1/BarrenLands_image/{pathlib.Path(x).name}"

def mask_url(x):
    return f"https://ragatesitng-dev-storage.s3.ap-south-1.amazonaws.com/1/BarrenLands/{pathlib.Path(x).name}"

def attached_label(id, loss):
    return {id:loss}

def merge_loss_dicts(loss_dicts):
    mistake_score = MistakeScore()
    for d in loss_dicts:
        for key, value in d.items():
            mistake_score.add(key=key, value=value)
    return mistake_score

def csv_parser(file_path):
    df = pd.read_csv(file_path)
    data_frame = pd.DataFrame()
    data_frame["ImageId"] = df["ImageId"].apply(lambda x: StringElement(x))
    data_frame["ImageUri"] = df["SourceLink"].apply(lambda x: StringElement(image_url(x)))
    data_frame["TimeOfCapture"] = df.apply(lambda row: TimeStampElement(get_timestamp_x_hours_ago(row.name)), axis=1)
    data_frame["SourceLink"] = df['SourceLink'].apply(lambda x:StringElement(x))
    data_frame["Annotations"] = df['Annotations'].apply(lambda x:StringElement(mask_url(x)))
    return data_frame
data_frame = csv_parser("./assets/BarrenLands.csv")


schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement())
schema.add("ImageUri", ImageUriSchemaElement())
schema.add("TimeOfCapture", TimeOfCaptureSchemaElement())
schema.add("SourceLink", FeatureSchemaElement())
schema.add("Annotations", TIFFSchemaElement(label_mapping=label_to_classname, schema="tiff"))


run_name = f"loader_lq_ss-drift-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"


test_session = TestSession(project_name="Semantic Segmentation", run_name = run_name)

cred = DatasetCreds(region="ap-south-1")

#create test_ds object of Dataset instance
test_ds = Dataset(test_session=test_session, 
                  name="BarrenLands-final", 
                  type=DATASET_TYPE.IMAGE,
                  data=data_frame, 
                  schema=schema, 
                  creds=cred)

# #load schema and pandas data frame
test_ds.load()


model_exe_fun = ModelExecutorFactory().get_model_executor(test_session=test_session, 
                                                          model_name="Satsure Embedding Model", 
                                                          version="0.1.1")

df = model_exe_fun.execute(init_args={"device": "cpu"}, 
                           execution_args={"input_columns":{"img_paths":"ImageUri"}, 
                                           "output_columns":{"embedding":"ImageEmbedding"},
                                           "column_schemas":{"embedding":ImageEmbeddingSchemaElement(model="Satsure Embedding Model")}}, 
                           data_frame=test_ds)

test_ds.load()


from raga import *
import datetime


run_name = f"lm-inference-test-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"


# create test_session object of TestSession instance
test_session = TestSession(project_name="testingProject", run_name= run_name, access_key="LGXJjQFD899MtVSrNHGH", secret_key="TC466Qu9PhpOjTuLu5aGkXyGbM7SSBeAzYH6HpcP", host="http://3.111.106.226:8080")


filters = Filter()
filters.add(TimestampFilter(gte="2020-03-15T00:00:00Z", lte="2050-03-15T00:00:00Z"))

# #Model: Production-Vienna-Alto
# #Version: 0.0.1
# lightmetrics_inference_generator(test_session=test_session, 
#                    dataset_name="test-lm-loader-3-nov-v2", 
#                    filter=filters, 
#                    model_name="Production-Vienna-Alto-0.0.1", 
#                    event_inference_col_name="Production-Vienna-Alto-Event", 
#                    model_inference_col_name="Production-Vienna-Alto-Model")

# #Model: Production-Canada-Stop-Quebec
# #Version: 0.0.1
# lightmetrics_inference_generator(test_session=test_session, 
#                    dataset_name="test-lm-loader-3-nov-v2", 
#                    filter=filters, 
#                    model_name="Production-Canada-Stop-Quebec-0.0.1", 
#                    event_inference_col_name="Production-Canada-Stop-Quebec-Event", 
#                    model_inference_col_name="Production-Canada-Stop-Quebec-Model")

#Model: Production-America-Stop
#Version: 0.0.1
lightmetrics_inference_generator(test_session=test_session, 
                   dataset_name="test-lm-loader-3-nov-v3", 
                   filter=filters, 
                   model_name="Production-America-Stop-0.0.1", 
                   event_inference_col_name="Production-America-Stop-Event", 
                   model_inference_col_name="Production-America-Stop-Model")

# #Model: Production-Vienna-Stop
# #Version: 0.0.1
# lightmetrics_inference_generator(test_session=test_session, 
#                    dataset_name="test-lm-loader-3-nov-v2", 
#                    filter=filters, 
#                    model_name="Production-Vienna-Stop-0.0.1", 
#                    event_inference_col_name="Production-Vienna-Stop-Event", 
#                    model_inference_col_name="Production-Vienna-Stop-Model")


# #Model: Complex-Vienna-Alto
# #Version: 0.0.1
# lightmetrics_inference_generator(test_session=test_session, 
#                    dataset_name="test-lm-loader-3-nov-v2", 
#                    filter=filters, 
#                    model_name="Complex-Vienna-Alto-0.0.1", 
#                    event_inference_col_name="Complex-Vienna-Alto-Event", 
#                    model_inference_col_name="Complex-Vienna-Alto-Model")

# #Model: Complex-Canada-Stop-Quebec
# #Version: 0.0.1
# lightmetrics_inference_generator(test_session=test_session, 
#                    dataset_name="test-lm-loader-3-nov-v2", 
#                    filter=filters, 
#                    model_name="Complex-Canada-Stop-Quebec-0.0.1", 
#                    event_inference_col_name="Complex-Canada-Stop-Quebec-Event", 
#                    model_inference_col_name="Complex-Canada-Stop-Quebec-Model")

# #Model: Complex-America-Stop
# #Version: 0.0.1
# lightmetrics_inference_generator(test_session=test_session, 
#                    dataset_name="test-lm-loader-3-nov-v2", 
#                    filter=filters, 
#                    model_name="Complex-America-Stop-0.0.1", 
#                    event_inference_col_name="Complex-America-Stop-Event", 
#                    model_inference_col_name="Complex-America-Stop-Model")

# #Model: Complex-Vienna-Stop
# #Version: 0.0.1
# lightmetrics_inference_generator(test_session=test_session, 
#                    dataset_name="test-lm-loader-3-nov-v2", 
#                    filter=filters, 
#                    model_name="Complex-Vienna-Stop-0.0.1", 
#                    event_inference_col_name="Complex-Vienna-Stop-Event", 
#                    model_inference_col_name="Complex-Vienna-Stop-Model")


from raga import *
import datetime

run_name = f"drift-test-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"

# create test_session object of TestSession instance
test_session = TestSession(project_name="Satellite Imagery", run_name = run_name, profile="prod")

rules = DriftDetectionRules()

# rules.add(type="multi_class_anomaly_detection", dist_metric="Mahalanobis", _class="ALL")
# edge_case_detection = data_drift_detection(test_session=test_session,
#                                            test_name="Drift-detection-test-HW_OD",
#                                            train_dataset_name="drift_honeywell_traffic_gt_object_detection_v2",
#                                            field_dataset_name="drift_honeywell_traffic_pred_object_detection",
#                                            train_embed_col_name="ImageVectorsM1",
#                                            field_embed_col_name = "ImageVectorsM1",
#                                            output_type = "multi_class_object_detection",
#                                            level = "image",
#                                            rules = rules)

# edge_case_detection = data_drift_detection(test_session=test_session,
#                                            test_name="Drift-detection-test",
#                                            train_dataset_name="grassLands-v1",
#                                            field_dataset_name="barrenland_v1",
#                                            train_embed_col_name="Annotations",
#                                            field_embed_col_name = "Annotations",
#                                            output_type = "semantic_segmentation",
#                                            level = "image",
#                                            aggregation_level=["location", "vehicle_no", "channel"],
#                                            rules = rules)

# edge_case_detection = data_drift_detection(test_session=test_session,
#                                            test_name="Drift-detection-test",
#                                            train_dataset_name="grasslands-v1",
#                                            field_dataset_name="barrenlands-v1",
#                                            train_embed_col_name="ImageVectorsM1",
#                                            field_embed_col_name = "ImageVectorsM1",
#                                            output_type = "semantic_segmentation",
#                                            level = "image",
#                                            rules = rules)

# edge_case_detection = data_drift_detection(test_session=test_session,
#                                            test_name="Drift-detection-test",
#                                            dataset_name="super_resolution_data_v1",
#                                            embed_col_name="imageEmbedding",
#                                            output_type = "super_resolution",
#                                            rules = rules)

# rules.add(type="anomaly_detection", dist_metric="Mahalanobis", _class="ALL", threshold=21.3)
# edge_case_detection = data_drift_detection(test_session=test_session,
#                                            test_name="Drift-detection-test_AI_dash",
#                                            dataset_name="drift_ai_dash_pred_object_detection_ds-10-v3_ai_dash_t21",
#                                            embed_col_name = "ImageVectorsM1",
#                                            output_type = "outlier_detection",
#                                            rules = rules)

# # add payload into test_session object
# test_session.add(edge_case_detection)
# #run added test
# test_session.run()


# project_name = "Semantic Segmentation - Images"
# run_name = f"Demo-Run-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"
# test_session = TestSession(project_name="Semantic Segmentation - Images", run_name= run_name, access_key="DlGhu2Y9D7Kbc955Y0tz", secret_key="euSZgU49jkIV4CTekcKvpqdk93sQpZdRe338ym1N", host="http://13.126.220.245:8080")

# rules = DriftDetectionRules()
# rules.add(type="anomaly_detection", dist_metric="Mahalanobis", _class="ALL", threshold=21.0)

# edge_case_detection = data_drift_detection(test_session=test_session,
#                                            test_name="Drift-detection-test",
#                                            train_dataset_name="grasslands-final",
#                                            field_dataset_name="barrenlands-final",
#                                            train_embed_col_name="ImageEmbedding",
#                                            field_embed_col_name = "ImageEmbedding",
#                                            output_type = "semantic_segmentation",
#                                            level = "image",
#                                            rules = rules)

rules = DriftDetectionRules()
rules.add(type="anomaly_detection", dist_metric="Mahalanobis", _class="ALL", threshold=21.0)

edge_case_detection = data_drift_detection(test_session=test_session,
                                           test_name="Drift-detection-test",
                                           train_dataset_name="grasslands-final",
                                           field_dataset_name="barrenlands-final",
                                           train_embed_col_name="ImageEmbedding",
                                           field_embed_col_name = "ImageEmbedding",
                                           output_type = "semantic_segmentation",
                                           level = "image",
                                           rules = rules)


test_session.add(edge_case_detection)

test_session.run()




from raga import *

# create test_session object of TestSession instance
test_session = TestSession(project_name="testingProject", run_name= "LM-Model-Upload-Run-Test-v1", access_key="LGXJjQFD899MtVSrNHGH", secret_key="TC466Qu9PhpOjTuLu5aGkXyGbM7SSBeAzYH6HpcP", host="http://3.111.106.226:8080")

lightmetrics_model_upload(test_session=test_session, file_path="/home/ubuntu/developments/testing-platform-python-client/raga/examples/assets/lm_models/Production-America-Stop.zip", name="Production-America-Stop", version="0.0.1")
lightmetrics_model_upload(test_session=test_session, file_path="/home/ubuntu/developments/testing-platform-python-client/raga/examples/assets/lm_models/Production-Canada-Stop-Quebec.zip", name="Production-Canada-Stop-Quebec", version="0.0.1")
lightmetrics_model_upload(test_session=test_session, file_path="/home/ubuntu/developments/testing-platform-python-client/raga/examples/assets/lm_models/Production-Vienna-Alto.zip", name="Production-Vienna-Alto", version="0.0.1")
lightmetrics_model_upload(test_session=test_session, file_path="/home/ubuntu/developments/testing-platform-python-client/raga/examples/assets/lm_models/Production-Vienna-Stop.zip", name="Production-Vienna-Stop", version="0.0.1")


# lightmetrics_model_upload(test_session=test_session, file_path="/home/ubuntu/developments/testing-platform-python-client/raga/examples/assets/lm_models/Complex-America-Stop_Config_test_missing.zip", name="Complex-America-Stop_Config_test_missing", version="0.0.1")

# lightmetrics_model_upload(test_session=test_session, file_path="/home/ubuntu/developments/testing-platform-python-client/raga/examples/assets/lm_models/Complex-America-Stop_executable_missing.zip", name="Complex-America-Stop_executable_missing", version="0.0.1")

# lightmetrics_model_upload(test_session=test_session, file_path="/home/ubuntu/developments/testing-platform-python-client/raga/examples/assets/lm_models/Complex-America-Stop_Config_test_missmatch_model_name.zip", name="Complex-America-Stop_Config_test_missmatch_model_name", version="0.0.1")

# lightmetrics_model_upload(test_session=test_session, file_path="/home/ubuntu/developments/testing-platform-python-client/raga/examples/assets/lm_models/Complex-America-Stop-DLC_test.zip", name="Complex-America-Stop-DLC_test", version="0.0.1")



import pathlib
from raga import *
import pandas as pd


def image_url(x):
    return f"https://ragatesitng-dev-storage.s3.ap-south-1.amazonaws.com/1/BarrenLands_image/{pathlib.Path(x).name}"


def csv_parser(file_path):
    df = pd.read_csv(file_path).head(5)
    data_frame = pd.DataFrame()
    data_frame["ImageId"] = df["ImageId"].apply(lambda x: StringElement(x))
    data_frame["ImageUri"] = df["SourceLink"].apply(lambda x: StringElement(image_url(x)))
    return data_frame

data_frame = csv_parser("./assets/BarrenLands.csv")


schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement())
schema.add("ImageUri", ImageUriSchemaElement())

test_session = TestSession(project_name="testingProject", 
                           run_name= "Embedding Generator Run Test", 
                           profile="dev")

#create test_ds object of Dataset instance
test_ds = Dataset(test_session=test_session, 
                  name="model-distribution-test-v1", 
                  type=DATASET_TYPE.IMAGE,
                  data=data_frame, 
                  schema=schema)

# #load schema and pandas data frame
test_ds.load()


model_exe_fun = ModelExecutorFactory().get_model_executor(test_session=test_session, 
                                                          model_name="Satsure Embedding Model", 
                                                          version="0.1.1")

df = model_exe_fun.execute(init_args={"device": "cpu"}, 
                           execution_args={"input_columns":{"img_paths":"ImageUri"}, 
                                           "output_columns":{"embedding":"ImageEmbedding"},
                                           "column_schemas":{"embedding":ImageEmbeddingSchemaElement(model="Satsure Embedding Model")}}, 
                           data_frame=test_ds)

test_ds.load()


from abc import ABC
from typing import Optional

class CoreFilter(ABC):
    def __init__(self):
        pass

class Filter():
    def __init__(self):
        self.filters = {}
    
    def add(self, filter:CoreFilter):
        self.filters.update(filter.__dict__)
        return self.filters
    
    def get(self):
        return self.filters
    
class TimestampFilter(CoreFilter):
    def __init__(self, gte:Optional[str]="", lte:Optional[str]=""):
        super().__init__()
        self.timestamp = {
            "gte": gte,
            "lte":lte   
        }



import ast
import pathlib
import random
from raga import *
import pandas as pd
import datetime

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def replace_img_url(old_value):
    url = f"https://ragatesitng-dev-storage.s3.ap-south-1.amazonaws.com/1/{old_value}"
    return StringElement(url)

def replace_label_url(old_value):
    url = f"https://ragatesitng-dev-storage.s3.ap-south-1.amazonaws.com/1/{old_value}"
    return StringElement(url)

def replace_embedding(old_value):
    embeddings = ImageEmbedding()
    embeddings_list = ast.literal_eval(old_value)
    for embedding in embeddings_list:
        embeddings.add(Embedding(embedding))
    return embeddings

label_to_classname = {
    1: "unhealthy"
}

def merge_loss_dicts(x):
    mistake_score = MistakeScore()
    mistake_score.add(key=1, value=0, area=0)  
    return mistake_score

def csv_parser(csv_path):
    df = pd.read_csv(csv_path)
    data_frame = pd.DataFrame()
    data_frame["ImageId"] = df["label"].apply(lambda x: StringElement(pathlib.Path(x).stem))
    data_frame["ImageUri"] = df['rgb_img_tif'].apply(lambda x: replace_img_url(x))
    data_frame["TimeOfCapture"] = df.apply(lambda row: TimeStampElement(get_timestamp_x_hours_ago(row.name)), axis=1)
    data_frame["Reflection"] = df.apply(lambda row: StringElement(random.choice(["Yes", "No"])), axis=1)
    data_frame["Overlap"] = df.apply(lambda row: StringElement(random.choice(["Yes", "No"])), axis=1)
    data_frame["CameraAngle"] = df.apply(lambda row: StringElement(random.choice(["Yes", "No"])), axis=1)
    data_frame["Annotations"] = df['label'].apply(lambda x: replace_label_url(x))
    data_frame["MistakeScores"] = df['label'].apply(merge_loss_dicts)
    return data_frame


pd_data_frame = csv_parser("./assets/tree_health_segmentation.csv")


schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement())
schema.add("ImageUri", ImageUriSchemaElement())
schema.add("TimeOfCapture", TimeOfCaptureSchemaElement())
schema.add("Reflection", AttributeSchemaElement())
schema.add("Overlap", AttributeSchemaElement())
schema.add("CameraAngle", AttributeSchemaElement())
schema.add("Annotations", TIFFSchemaElement(label_mapping=label_to_classname, schema="tiff", model="ModelA"))
schema.add("MistakeScores", MistakeScoreSchemaElement(ref_col_name="Annotations"))




run_name = f"loader_failure_mode_analysis_semantic_segmentation-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"

# create test_session object of TestSession instance
# test_session = TestSession(project_name="testingProject", run_name= run_name, access_key="LGXJjQFD899MtVSrNHGH", secret_key="TC466Qu9PhpOjTuLu5aGkXyGbM7SSBeAzYH6HpcP", host="http://3.111.106.226:8080")

# test_session = TestSession(project_name="SatelliteImagery", run_name= run_name, access_key="XGgIC7lSNgoP5U3UtLjs", secret_key="vHWY3ENWN56Sbv8BoKyW3dQWXcDDEOAjL1YU6s3M", host="http://13.126.220.245:8080")
test_session = TestSession(project_name="Semantic Segmentation - Images", run_name= run_name, access_key="DlGhu2Y9D7Kbc955Y0tz", secret_key="euSZgU49jkIV4CTekcKvpqdk93sQpZdRe338ym1N", host="http://13.126.220.245:8080")

cred = DatasetCreds(region="ap-south-1")

#create test_ds object of Dataset instance
test_ds = Dataset(test_session=test_session,
                  name="tree_health_segmentation_dataset",
                  type=DATASET_TYPE.IMAGE,
                  data=pd_data_frame,
                  schema=schema,
                  creds=cred)

#load to server
test_ds.load()



import pandas as pd
import json
import pathlib
from raga import *
import pandas as pd
import datetime

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def img_url(x):
    parts = x.split("-")
    parts.pop()
    video = "-".join(parts)
    image = x.split("-")
    number = image[-1]
    image.pop()
    image = "-".join(parts)
    return StringElement(f"https://ragacloudstorage.s3.ap-south-1.amazonaws.com/1/Hb.json/data_points/{video}/images/img_{image}_{number}")

def annotation_v1(row):
    AnnotationsV1 = ImageDetectionObject()
    for index, detection in enumerate(row["detections"]):
        AnnotationsV1.add(ObjectDetection(Id=index, ClassId=0, ClassName=detection['class'], Confidence=detection['confidence'], BBox= detection['bbox'], Format="xywh_normalized"))
    return AnnotationsV1

def model_inferences(row):
    ModelInferences = ImageDetectionObject()
    for index, detection in enumerate(row["detections"]):
        ModelInferences.add(ObjectDetection(Id=index, ClassId=0, ClassName=detection['class'], Confidence=detection['confidence'], BBox= detection['bbox'], Format="xywh_normalized"))
    return ModelInferences

def imag_vectors_m1(row):
    ImageVectorsM1 = ImageEmbedding()
    for embedding in row["ImageVectorsM1"]:
        ImageVectorsM1.add(Embedding(embedding))
    return ImageVectorsM1

def roi_vectors_m1(row):
    ROIVectorsM1 = ROIEmbedding()
    for embedding in row["ROIVectorsM1"]:
        ROIVectorsM1.add(id=embedding.get("Id"), embedding_values=embedding.get("embedding"))
    return ROIVectorsM1

def make_video_id(x):
    return StringElement(pathlib.Path(x).name)


def make_frame_number(x):
    x_split = x.split("_")
    frame = pathlib.Path(x_split[-1]).stem
    return StringElement(int(frame))

# Replace 'your_json_file.json' with the path to your JSON file
json_file_path = './assets/embeddings.json'

# Open the JSON file and read its contents
with open(json_file_path, 'r') as json_file:
    data = json.load(json_file)

videos = []
videos_embd = []
for item in data:
    video = item['inputs'][0]
    if video not in videos:
        videos.append(item)

emb = pd.DataFrame(videos)
modela = pd.read_json("./assets/model1.json")
modelb = pd.read_json("./assets/model2.json")
emb = emb.explode('inputs')
modela = modela.explode('inputs')
modelb = modelb.explode('inputs')


# Merge the DataFrames on 'key' column one by one
merged_df = pd.merge(emb, modela, on='inputs', suffixes=('_emb','_modela'))
df = pd.merge(merged_df, modelb, on='inputs', suffixes=('_org', '_modelb'))

# df.head(10).to_json("./assets/lm_emb.json")


data_frame = pd.DataFrame()
data_frame["ImageId"] = df["outputs_emb"].apply(lambda x: StringElement(x[0].get('imageName')))
data_frame["Frame"] = df["outputs_emb"].apply(lambda x: make_frame_number(x[0].get('imageName')))
data_frame["VideoId"] = df["inputs"].apply(lambda x: make_video_id(x))
data_frame["ImageUri"] = df["outputs_emb"].apply(lambda x: img_url(x[0].get('imageName')))
data_frame["SourceLink"] = df["outputs_emb"].apply(lambda x: StringElement(x[0].get('imageName')))
data_frame["TimeOfCapture"] = df.apply(lambda row: TimeStampElement(get_timestamp_x_hours_ago(row.name)), axis=1)
data_frame["weather"] = df["attributes_org"].apply(lambda x: StringElement(x.get("weather")))
data_frame["time_of_day"] = df["attributes_org"].apply(lambda x: StringElement(x.get("time_of_day")))
data_frame["scene"] = df["attributes_org"].apply(lambda x: StringElement(x.get("scene")))
data_frame["ModelA"] = df.apply(lambda x: annotation_v1(x), axis=1)
# data_frame["ModelB"] = df["outputs"].apply(lambda x: model_inferences(x))
# data_frame["ImageVectorsM1"] = df["outputs_emb"].apply(lambda x: imag_vectors_m1(x))
print(data_frame)





import pathlib
import re
from raga import *
import pandas as pd
import datetime

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def img_url(x):
    parts = x.split("-")
    parts.pop()
    video = "-".join(parts)
    image = x.split("-")
    number = image[-1]
    image.pop()
    image = "-".join(parts)
    return StringElement(f"https://ragacloudstorage.s3.ap-south-1.amazonaws.com/1/Hb.json/data_points/{video}/images/img_{image}_{number}")

def annotation_v1(row):
    AnnotationsV1 = ImageDetectionObject()
    for detection in row["AnnotationsV1"]:
        AnnotationsV1.add(ObjectDetection(Id=detection['Id'], ClassId=detection['ClassId'], ClassName=detection['ClassName'], Confidence=detection['Confidence'], BBox= detection['BBox'], Format="xywh_normalized"))
    return AnnotationsV1

def model_inferences(row):
    ModelInferences = ImageDetectionObject()
    for detection in row["ModelInferences"]:
        ModelInferences.add(ObjectDetection(Id=detection['Id'], ClassId=detection['ClassId'], ClassName=detection['ClassName'], Confidence=detection['Confidence'], BBox= detection['BBox'], Format="xywh_normalized"))
    return ModelInferences

def imag_vectors_m1(row):
    ImageVectorsM1 = ImageEmbedding()
    for embedding in row["ImageVectorsM1"]:
        ImageVectorsM1.add(Embedding(embedding))
    return ImageVectorsM1

def roi_vectors_m1(row):
    ROIVectorsM1 = ROIEmbedding()
    for embedding in row["ROIVectorsM1"]:
        ROIVectorsM1.add(id=embedding.get("Id"), embedding_values=embedding.get("embedding"))
    return ROIVectorsM1

def make_video_id(x):
    x_split = x.split("-")
    x_split.pop()
    video = "-".join(x_split)
    return StringElement(video)


def make_frame_number(x):
    x_split = x.split("-")
    frame = pathlib.Path(x_split[-1]).stem
    return StringElement(int(frame))

def json_parser(json_file):
    df = pd.read_json(json_file)
    data_frame = pd.DataFrame()
    data_frame["ImageId"] = df["ImageId"].apply(lambda x: StringElement(x))
    data_frame["Frame"] = df["ImageId"].apply(lambda x: make_frame_number(x))
    data_frame["VideoId"] = df["ImageId"].apply(lambda x: make_video_id(x))
    data_frame["ImageUri"] = df["ImageId"].apply(lambda x: img_url(x))
    data_frame["SourceLink"] = df["ImageId"].apply(lambda x: StringElement(x))
    data_frame["TimeOfCapture"] = df.apply(lambda row: TimeStampElement(get_timestamp_x_hours_ago(row.name)), axis=1)
    data_frame["weather"] = df["weather"].apply(lambda x: StringElement(x))
    data_frame["time_of_day"] = df["time_of_day"].apply(lambda x: StringElement(x))
    data_frame["scene"] = df["scene"].apply(lambda x: StringElement(x))
    data_frame["ModelA"] = df.apply(annotation_v1, axis=1)
    data_frame["ModelB"] = df.apply(model_inferences, axis=1)
    data_frame["ImageVectorsM1"] = df.apply(imag_vectors_m1, axis=1)
    # data_frame["ROIVectorsM1"] = df.apply(roi_vectors_m1, axis=1)
    return data_frame


pd_data_frame = json_parser("./assets/resultframe4.json")



#### Want to see in csv file uncomment line below ####
# data_frame_extractor(pd_data_frame).to_csv("./assets/resultframe4.csv", index=False)

schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement())
schema.add("Frame", FrameSchemaElement())
schema.add("VideoId", ParentSchemaElement())
schema.add("ImageUri", ImageUriSchemaElement())
schema.add("TimeOfCapture", TimeOfCaptureSchemaElement())
schema.add("SourceLink", FeatureSchemaElement())
schema.add("weather", AttributeSchemaElement())
schema.add("time_of_day", AttributeSchemaElement())
schema.add("scene", AttributeSchemaElement())
schema.add("ModelA", InferenceSchemaElement(model="modelA"))
schema.add("ModelB", InferenceSchemaElement(model="modelB"))
# schema.add("ROIVectorsM1", RoiEmbeddingSchemaElement(model="ROIModel"))
schema.add("ImageVectorsM1", ImageEmbeddingSchemaElement(model="imageModel"))

run_name = f"lm_image_loader_failure_mode_analysis-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"

# create test_session object of TestSession instance
test_session = TestSession(project_name="testingProject", run_name= run_name, access_key="LGXJjQFD899MtVSrNHGH", secret_key="TC466Qu9PhpOjTuLu5aGkXyGbM7SSBeAzYH6HpcP", host="http://3.111.106.226:8080")

creds = DatasetCreds(region="ap-south-1")

test_ds = Dataset(test_session=test_session, 
                    name="lm-hb-image-ds-v12", 
                    type=DATASET_TYPE.IMAGE,
                    data=pd_data_frame, 
                    schema=schema, 
                    creds=creds)

#load schema and pandas data frame
test_ds.load()



# def count_rows_and_chunk(df, chunk_size):
#     """
#     Count the rows of a Pandas DataFrame and break it into smaller chunks.

#     Parameters:
#         df (pd.DataFrame): The input Pandas DataFrame.
#         chunk_size (int): The size of each chunk.

#     Returns:
#         (int, list of pd.DataFrame): A tuple containing the total row count of the DataFrame
#                                      and a list of DataFrame chunks.
#     """
#     total_rows = len(df)
#     num_chunks = (total_rows + chunk_size - 1) // chunk_size  # Calculate the number of chunks needed
#     chunks = [df[i * chunk_size:(i + 1) * chunk_size] for i in range(num_chunks)]
#     return total_rows, chunks


# chunk_size = 5000  # Maximum chunk size

# total_rows, chunks = count_rows_and_chunk(pd_data_frame, chunk_size)
# print(f"Total Rows: {total_rows}")
# for i, chunk in enumerate(chunks):
#    chunk.to_pickle(f"./assets/lm_data_2/lm_data_{str(i).zfill(8)}.pkl")


# paths = []
# folder_path = "./assets/lm_data_2"
# # Check if the folder path exists
# if os.path.exists(folder_path) and os.path.isdir(folder_path):
#     # Get a list of all files in the folder
#     file_list = [f for f in os.listdir(folder_path) if os.path.isfile(os.path.join(folder_path, f))]
    
#     # Print the list of files with their full paths
#     for file_name in file_list:
#         full_path = os.path.join(folder_path, file_name)
#         paths.append(full_path)
# else:
#     print(f"The folder path '{folder_path}' does not exist or is not a directory.")
# paths = sorted(paths)
# # print(paths)
# for path in paths:
#     # create test_ds object of Dataset instance
#     df = pd.read_pickle(path)
#     print("PAAAAAATH", path)
#     test_ds = Dataset(test_session=test_session, 
#                       name="lm-hb-image-ds-v13", 
#                       type=DATASET_TYPE.IMAGE,
#                       data=df, 
#                       schema=schema, 
#                       creds=creds)

#     #load schema and pandas data frame
#     test_ds.load()


from raga import *
import pandas as pd
import datetime
import ast

label_to_attribute = {
    0 : 'total',
    1 : 'merchant_name',
    2 : 'date',
    3 : 'NA'
}

def inferenceCount(x):
    inferenceCount = InferenceCount()
    x = ast.literal_eval(x)
    for key, value in x.items():
        inferenceCount.add(key, value)
    return inferenceCount

def detection(row):
    Detection = ImageDetectionObject()
    row["Detection"] = ast.literal_eval(row["Detection"])
    for detection in row["Detection"]:
        Detection.add(ObjectDetection(Id=detection['Id'], ClassId=detection['ClassId'], ClassName=detection['ClassName'], Confidence=detection['Confidence'], BBox= detection['BBox'], Format="xywh_normalized"))
        return Detection

def recognition(row):
    Recognition = ImageDetectionObject()
    row["Recognition"] = ast.literal_eval(row["Recognition"])
    for recognition in row["Recognition"]:
        Recognition.add(ObjectRecognition(Id=recognition['Id'], ClassId=recognition['ClassId'], ClassName=recognition['ClassName'], Confidence=recognition['Confidence'], OcrText= recognition['OcrText']))
    return Recognition

def csv_parser(csv_file):
    df = pd.read_csv(csv_file)
    data_frame = pd.DataFrame()
    data_frame["ImageId"] = df["ImageId"].apply(lambda x: StringElement(x))
    data_frame["Detection"] = df.apply(detection, axis = 1)
    data_frame["Recognition"] = df.apply(recognition, axis = 1)
    data_frame["actual_detections"] = df["actual_detections"].apply(inferenceCount)

    return data_frame.head(5)

###################################################################################################
pd_data_frame = csv_parser("./assets/dataset1_annotations.csv")


schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement())
schema.add("Detection", InferenceSchemaElement(model="GT"))
schema.add("Recognition", ObjectRecognitionSchemaElement())
schema.add("actual_detections", InferenceCountSchemaElement(label_mapping=label_to_attribute))


run_name = f"Nano_Net_Dataset-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"

# create test_session object of TestSession instance
test_session = TestSession(project_name="testingProject", run_name= run_name, access_key="LGXJjQFD899MtVSrNHGH", secret_key="TC466Qu9PhpOjTuLu5aGkXyGbM7SSBeAzYH6HpcP", host="http://3.111.106.226:8080")

# cred = DatasetCreds(region="us-east-1")
cred = DatasetCreds(arn="arn:aws:iam::527593518644:role/raga-importer")


test_ds = Dataset(test_session=test_session,
                  name="Nano_Net_Dataset_3_nov_v2",
                  type=DATASET_TYPE.IMAGE,
                  data=pd_data_frame,
                  schema=schema,
                  creds=cred)

#load to server
test_ds.load()